"""
easy_logger.py

logger with already set up generalized file handlers

"""
import logging
from datetime import datetime
from typing import Union, List

from EasyLoggerAJM import _EasyLoggerCustomLogger, ColorizedFormatter, NO_COLORIZER
from EasyLoggerAJM.sub_initializers import (_PropertiesInitializer,
                                            _InternalLoggerMethods,
                                            _HandlerInitializer)


class EasyLoggerInitializer(_PropertiesInitializer,
                            _InternalLoggerMethods,
                            _HandlerInitializer):
    DEFAULT_FORMAT = '%(asctime)s | %(name)s | %(levelname)s | %(message)s'

    def __init__(self, project_name=None, chosen_format=DEFAULT_FORMAT, **kwargs):
        super().__init__(root_log_location=kwargs.get('root_log_location', None))
        self._chosen_format = chosen_format
        self._no_stream_color = kwargs.get('no_stream_color', False)

        self.show_warning_logs_in_console = kwargs.get('show_warning_logs_in_console', False)

        # this variable is to differentiate between the unpacked kwargs,
        # and the list of kwargs that need to be used
        # it is ONLY used in this one situation and should not be reused
        kwargs_passed_in = kwargs
        self._initialize_internal_logger(kwargs_passed_in, **kwargs)

        self._set_initial_properties_value(project_name=project_name, **kwargs)

        self.timestamp = kwargs.get('timestamp', self.log_spec['timestamp'])
        self._set_timestamp_if_different()

        self.formatter, self.stream_formatter = self._setup_formatters(**kwargs)

    def set_timestamp(self, **kwargs):
        """
        This method, `set_timestamp`, is a static method that can be used to set a timestamp for logging purposes.
        The method takes in keyword arguments as parameters.

        Parameters:
            **kwargs: Keyword arguments that can contain the following keys:
                - timestamp (datetime or str, optional): A datetime object or a string representing a timestamp.
                    By default, this key is set to None.

        Returns:
            str: Returns a string representing the set timestamp.

        Raises:
            AttributeError: If the provided timestamp is not a datetime object or a string.

        Notes:
            - If the keyword argument 'timestamp' is provided, the method will return the provided timestamp if it is a
                datetime object or a string representing a timestamp.
            - If the keyword argument 'timestamp' is not provided or is set to None, the method will generate a
                timestamp using the current date and time in ISO format without seconds and colons.

        Example:
            # Set a custom timestamp
            timestamp = set_timestamp(timestamp='2022-01-01 12:34')

            # Generate a timestamp using current date and time
            current_timestamp = set_timestamp()
        """
        timestamp = kwargs.get('timestamp', None)
        if timestamp is not None:
            if isinstance(timestamp, (datetime, str)):
                self._internal_logger.info(f"timestamp set to {timestamp}")
                return timestamp
            else:
                try:
                    raise AttributeError("timestamp must be a datetime object or a string")
                except AttributeError as e:
                    self._internal_logger.error(e, exc_info=True)
                    raise e from None
        else:
            timestamp = datetime.now().isoformat(timespec='minutes').replace(':', '')
            self._internal_logger.info(f"timestamp set to {timestamp}")
            return timestamp

    def _set_timestamp_if_different(self):
        """Set the timestamp if it's different from the log specification."""
        if self.timestamp != self._log_spec.get('timestamp'):
            self.timestamp = self.set_timestamp(timestamp=self.timestamp)

    def _setup_formatters(self, **kwargs) -> (logging.Formatter, Union[ColorizedFormatter, logging.Formatter]):
        formatter = kwargs.get('formatter', logging.Formatter(self._chosen_format))

        if not self._no_stream_color:
            stream_formatter = kwargs.get('stream_formatter', ColorizedFormatter(self._chosen_format))
        else:
            stream_formatter = kwargs.get('stream_formatter', logging.Formatter(self._chosen_format))
        return formatter, stream_formatter

    def _initialize_internal_logger(self, internal_loggable_attrs: dict, **kwargs):
        self._internal_logger = self._setup_internal_logger(verbose=kwargs.get('internal_verbose', False))

        self._log_attributes_internal(internal_loggable_attrs)
        self._internal_logger.info(f'show_warning_logs_in_console set to '
                                   f'{self.show_warning_logs_in_console}')


class EasyLogger(EasyLoggerInitializer):
    """

    EasyLogger
    ==========

    Class to provide an easy logging mechanism for projects.

    Attributes:
    -----------
    DEFAULT_FORMAT : str
        Default log format used in the absence of a specified format.

    INT_TO_STR_LOGGER_LEVELS : dict
        Mapping of integer logger levels to their string representations.

    STR_TO_INT_LOGGER_LEVELS : dict
        Mapping of string logger levels to their integer representations.

    MINUTE_LOG_SPEC_FORMAT : tuple
        Tuple representing the log specification format at minute granularity.

    MINUTE_TIMESTAMP : str
        Timestamp at minute granularity.

    HOUR_LOG_SPEC_FORMAT : tuple
        Tuple representing the log specification format at hour granularity.

    HOUR_TIMESTAMP : str
        Timestamp at hour granularity.

    DAILY_LOG_SPEC_FORMAT : str
        String representing the log specification format at daily granularity.

    DAILY_TIMESTAMP : str
        Timestamp at daily granularity.

    LOG_SPECS : dict
        Dictionary containing predefined logging specifications.

    Methods:
    --------
     __init__(self, project_name=None, root_log_location="../logs", chosen_format=DEFAULT_FORMAT, logger=None, **kwargs)
        Initialize EasyLogger instance with provided parameters.

    file_logger_levels(self)
        Property to handle file logger levels.

    project_name(self)
        Property method to get the project name.

    inner_log_fstructure(self)
        Get the inner log file structure.

    log_location(self)
        Get the log location for file handling.

    log_spec(self)
        Handle logging specifications.

    classmethod UseLogger(cls, **kwargs)
        Instantiate a class with a specified logger.

    Note:
    -----
    The EasyLogger class provides easy logging functionality for projects,
    allowing customization of log formats and levels.

    """
    SHOW_WARNING_LOGS_MSG = 'warning logs will be printed to console - creating stream handler'

    def __init__(self, logger=None, **kwargs):
        super().__init__(**kwargs)

        self.logger = self.initialize_logger(logger=logger)

        self.make_file_handlers()

        if self.show_warning_logs_in_console:
            self._internal_logger.info(self.__class__.SHOW_WARNING_LOGS_MSG)
            self.create_stream_handler(**kwargs)

        self.create_other_handlers()
        self.post_handler_setup()

    @staticmethod
    def _get_level_handler_string(handlers: List[logging.Handler]) -> str:
        return ', '.join([' - '.join((x.__class__.__name__, logging.getLevelName(x.level)))
                          for x in handlers])

    @classmethod
    def UseLogger(cls, **kwargs):
        """
        This method is a class method that can be used to instantiate a class with a logger.
        It takes in keyword arguments and returns an instance of the class with the specified logger.

        Parameters:
        - **kwargs: Keyword arguments that are used to instantiate the class.

        Returns:
        - An instance of the class with the specified logger.

        Usage:
            MyClass.UseLogger(arg1=value1, arg2=value2)

        Note:
            The logger used for instantiation is obtained from the `logging` module and is named 'logger'.
        """
        return cls(**kwargs, logger=kwargs.get('logger', None)).logger

    def _set_logger_class(self, logger_class=_EasyLoggerCustomLogger, **kwargs):
        self._internal_logger.info('no passed in logger detected')
        logging.setLoggerClass(logger_class)
        self._internal_logger.info(f'logger class set to \'{logger_class.__name__}\'')
        # Create a logger with a specified name
        self.logger = logging.getLogger(kwargs.get('logger_name', 'logger'))
        self._internal_logger.info(f'logger created with name set to \'{self.logger.name}\'')
        return self.logger

    def initialize_logger(self, logger=None, **kwargs) -> Union[logging.Logger, _EasyLoggerCustomLogger]:
        """
        :param logger: The logger instance to initialize. If None, a new logger will be created using the internal method.
        :type logger: logging.Logger or None
        :param kwargs: Additional parameters to configure the logger, such as propagate settings.
        :type kwargs: dict
        :return: The initialized logger instance.
        :rtype: Union[logging.Logger, _EasyLoggerCustomLogger]

        THIS IS HOW TO FIX ISSUE WITH MULTIPLE LOGGING INSTANCES, override with this:
        self.logger = super().initialize_logger(logger=logger, **kwargs)
        self.logger.propagate = False
        return self.logger
        """
        if not logger:
            self.logger = self._set_logger_class(**kwargs)
        else:
            self._internal_logger.info(f'passed in logger ({logger}) detected')
            self.logger: logging.getLogger = logger
        self.logger.propagate = kwargs.get('propagate', True)
        self._internal_logger.info('logger initialized')
        self._internal_logger.info(f'propagate set to {self.logger.propagate}')
        return self.logger

    def post_handler_setup(self):
        # set the logger level back to DEBUG, so it handles all messages
        self.logger.setLevel(10)
        self._internal_logger.info(f'logger level set back to {self.logger.level}')
        self.logger.info(f"Starting {self.project_name} with the following handlers: "
                         f"{self._get_level_handler_string(self.logger.handlers)}")
        if not self._no_stream_color and NO_COLORIZER:
            self.logger.warning("colorizer not available, logs may not be colored as expected.")
        self._internal_logger.info("final logger initialized")
        # print("logger initialized")


if __name__ == '__main__':
    el = EasyLogger(internal_verbose=True,
                    show_warning_logs_in_console=True)#, log_level_to_stream=logging.INFO)
    el.logger.info("this is an info message",
                   print_msg=True)



import pathlib
import unittest
from unittest import mock
from unittest.mock import patch, Mock, MagicMock

from EasyLoggerAJM import OutlookEmailHandler


@unittest.skip("Skipping OutlookEmailHandler tests, they are not ready")
class TestOutlookEmailHandler(unittest.TestCase):

    def setUp(self):
        # Provide required arguments for OutlookEmailHandler's parent class constructor
        email_msg = self.get_mock_outlook_email()  # Assuming this could be configured as needed
        logger_dir_path = pathlib.Path("/path/to/logs")  # Placeholder directory path
        recipient = "recipient@test.com"

        self.handler = OutlookEmailHandler(email_msg, logger_dir_path, recipient, dev_mode=True)
        self.record = mock.Mock()

    @patch("win32com.client.Dispatch")
    def get_mock_outlook_email(self, mock_dispatch):
        # Create a mock for the mail item
        mock_mail_item = MagicMock()
        mock_dispatch.return_value.CreateItem.return_value = mock_mail_item

        # Simulate handling Outlook interaction
        import win32com.client
        outlook = win32com.client.Dispatch("Outlook.Application")
        mail_item = outlook.CreateItem(0)  # Equivalent to creating a new mail item
        mail_item.To = "test@recipient.com"  # Mock setting the recipient
        mail_item.Subject = "Test Subject"  # Mock setting the subject
        mail_item.Body = "This is a test email."  # Mock setting the body
        #mail_item.Send()  # Mock sending the email

        # Assertions to ensure the expected interactions occurred
        #mock_dispatch.assert_called_once_with("Outlook.Application")
        mock_mail_item.To = "test@recipient.com"
        mock_mail_item.Subject = "Test Subject"
        mock_mail_item.Body = "This is a test email"
        return mock_mail_item
        #mock_mail_item.Send.assert_called_once()  # Ensures the 'Send' method was called

    @patch("EasyLoggerAJM.handlers.ZipFile", autospec=True)
    @patch("pathlib.Path", autospec=True)
    def test_emit(self, mock_path, mock_zip):
        print(self.handler.email_msg)
        self.record.levelname = "Error"
        self.handler.project_name = "ProjectName"
        self.handler.format = Mock()
        self.handler.recipient = "recipient@test.com"

        mock_path.is_file.return_value = True
        mock_path.resolve.return_value = pathlib.Path('mock_resolved_path')

        with patch.object(self.handler, '_prep_logfile_attachment') as mock_prep_func:
            mock_prep_func.return_value = (mock_path, mock_path)

            self.handler.emit(self.record)
            self.handler.format.assert_called_once_with(self.record)  # Ensure format is called
            self.handler._prep_logfile_attachment.assert_called_once()

            self.handler._cleanup_logfile_zip.assert_called_once_with(mock_path, mock_path)
            self.assertEqual(self.handler.email_msg.To, self.handler.recipient)
            self.assertTrue(mock_path.is_file.called)
            self.assertTrue(mock_path.resolve.called)

    def test_emit_with_exception(self):
        self.record.levelname = "Error"
        self.handler.project_name = "ProjectName"
        self.handler.format = Mock()
        self.handler.recipient = "recipient@test.com"

        with patch('builtins.print') as mocked_print:
            with patch.object(self.handler, '_prep_logfile_attachment') as mock_prep_func:
                mock_prep_func.side_effect = Exception("Test exception")

                self.handler.emit(self.record)
                self.handler.format.assert_called_once_with(self.record)  # Ensure format is called
                self.handler._prep_logfile_attachment.assert_called_once()
                mocked_print.assert_called_once_with('Error sending email: Test exception')


if __name__ == "__main__":
    unittest.main()



from EasyLoggerAJM.errs import InvalidEmailMsgType
from EasyLoggerAJM.custom_loggers import _EasyLoggerCustomLogger
from EasyLoggerAJM.handlers import _BaseCustomEmailHandler, OutlookEmailHandler, StreamHandlerIgnoreExecInfo
from EasyLoggerAJM.formatters import ColorizedFormatter, NO_COLORIZER
from EasyLoggerAJM.filters import ConsoleOneTimeFilter
from EasyLoggerAJM.easy_logger import EasyLogger

__all__ = ['_EasyLoggerCustomLogger', 'InvalidEmailMsgType', 'OutlookEmailHandler', 'StreamHandlerIgnoreExecInfo',
           'ColorizedFormatter', 'ConsoleOneTimeFilter', 'EasyLogger', 'NO_COLORIZER']



class InvalidEmailMsgType(Exception):
    DEFAULT_MSG = "email_msg must be one of {valid_msg_types} not {given_value}."
    _MISSING_ATTR_ERR_MSG = 'if msg is not given, valid_msg_types and given_value must be given'

    def __init__(self, msg=None, **kwargs):
        valid_msg_types = kwargs.get('valid_msg_types', None)
        given_value = kwargs.get('given_value', None)
        if msg:
            self.message = msg
        if not msg and valid_msg_types and given_value:
            self.message = self.__class__.DEFAULT_MSG.format(valid_msg_types=valid_msg_types,
                                                             given_value=given_value)
        else:
            raise AttributeError(self.__class__._MISSING_ATTR_ERR_MSG)




import unittest
from unittest.mock import patch
from EasyLoggerAJM.easy_logger import EasyLogger, _EasyLoggerCustomLogger
from logging import StreamHandler, DEBUG, shutdown
from shutil import rmtree
from pathlib import Path
from io import StringIO


class TestEasyLoggerCustomLogger(unittest.TestCase):
    """
    TestEasyLoggerCustomLogger class represents a unit test case for the EasyLoggerCustomLogger class.
    It provides methods to test logging functionalities for different log levels
    such as info, debug, warning, error, and critical.
    The setUp method initializes the logger and log methods for testing.
    The _iter_subtests method iterates over log methods and asserts the expected behavior of logging messages.
    The test_print_msg_for_all_log_levels and test_not_print_msg_for_all_log_levels methods further test printing
     messages for all log levels when should_print flag is set to True and False respectively.
    """
    def setUp(self):
        self.logger = EasyLogger().UseLogger()  # ._EasyLoggerCustomLogger("TestLogger")
        self.log_methods = {
            'info': self.logger.info,
            'debug': self.logger.debug,
            'warning': self.logger.warning,
            'error': self.logger.error,
            'critical': self.logger.critical
        }
        self.should_print = True
        self._initialize_for_log_check()

    @classmethod
    def tearDownClass(cls):
        shutdown()
        TestEasyLoggerCustomLogger.remove_test_dirs()

    @staticmethod
    def remove_test_dirs():
        for x in Path('./').iterdir():
            if x.name.startswith('test_logs') and x.is_dir():
                rmtree(x, ignore_errors=True)
                print(f'{x} removed')

    def _initialize_for_log_check(self):
        self.log_capture_string = StringIO()
        self.log_handler = StreamHandler(self.log_capture_string)
        self.logger.addHandler(self.log_handler)
        self.logger.setLevel(DEBUG)

    def _iter_subtests(self, mock_print):
        log_msg = "Test message"
        for level, method in self.log_methods.items():
            with self.subTest(level=level):
                with patch.object(_EasyLoggerCustomLogger, '_print_msg',
                                  wraps=self.logger._print_msg) as mock_print_msg:
                    method(log_msg, print_msg=self.should_print)
                    mock_print_msg.assert_called_once_with(log_msg, print_msg=self.should_print)
                    if not self.should_print:
                        mock_print.assert_not_called()
                    else:
                        mock_print.assert_called_once_with(log_msg)

                    # Check if the log message was actually logged
                    self.assertIn(log_msg, self.log_capture_string.getvalue())
                    mock_print.reset_mock()

    @patch('builtins.print')
    def test_print_msg_for_all_log_levels(self, mock_print):
        self.should_print = True
        self._iter_subtests(mock_print)

    @patch('builtins.print')
    def test_not_print_msg_for_all_log_levels(self, mock_print):
        self.should_print = False
        self._iter_subtests(mock_print)


"""class TestEasyLoggerCustomLoggerDEPRECATED(unittest.TestCase):
    def setUp(self):
        self.logger = EasyLogger().UseLogger().logger  # ._EasyLoggerCustomLogger("TestLogger")
        self.log_methods = {
            'info': self.logger.info,
            'debug': self.logger.debug,
            'warning': self.logger.warning,
            'error': self.logger.error,
            'critical': self.logger.critical
        }

    @patch('builtins.print')
    def test_internal_print_call_within_print_msg(self, mock_print):
        with patch.object(_EasyLoggerCustomLogger, '_print_msg',
                          wraps=_EasyLoggerCustomLogger._print_msg) as mock_print_msg:
            # Call the info method to execute _print_msg
            self.logger.info("Test message", print_msg=True)

            # Assert that _print_msg was called within the scope
            mock_print_msg.assert_called_once_with("Test message", print_msg=True)
            # Now check that the print function was called within _print_msg
            mock_print.assert_called_once_with("Test message")

    @patch.object(_EasyLoggerCustomLogger, '_print_msg')
    def test_info_print(self, mock_print):
        self.logger.info("info_msg", stack_info=True, print_msg=True)
        mock_print.assert_called_once_with("info_msg", print_msg=True)

    @patch.object(_EasyLoggerCustomLogger, '_print_msg')
    def test_warning_print(self, mock_print):
        self.logger.warning("warning_msg", stack_info=True, print_msg=True)
        mock_print.assert_called_once_with("warning_msg", print_msg=True)

    @patch.object(_EasyLoggerCustomLogger, '_print_msg')
    def test_error_print(self, mock_print):
        self.logger.error("error_msg", stack_info=True, print_msg=True)
        mock_print.assert_called_once_with("error_msg", print_msg=True)

    @patch.object(_EasyLoggerCustomLogger, '_print_msg')
    def test_debug_print(self, mock_print):
        self.logger.debug("debug_msg", stack_info=True, print_msg=True)
        mock_print.assert_called_once_with("debug_msg", print_msg=True)

    @patch.object(_EasyLoggerCustomLogger, '_print_msg')
    def test_critical_print(self, mock_print):
        self.logger.critical("critical_msg", stack_info=True, print_msg=True)
        mock_print.assert_called_once_with("critical_msg", print_msg=True)"""


if __name__ == "__main__":
    unittest.main()
